from django.utils.translation import ugettext as _

EXTRA_FIELDS = 'extra-form-fields'
SQL_INJECTION = 'sql-injection-attempt'
XSS = 'xss-attempt'
SESSION_CHANGED = 'session-changed'

trans = {
    EXTRA_FIELDS: _('Attempt to process form with extra values'),
    SQL_INJECTION: _('Data looks that like SQL injection attempt'),
    XSS: _('Data that looks like XSS attempt'),
    SESSION_CHANGED: _('Session data changed'),
}

__all__ = [EXTRA_FIELDS, SQL_INJECTION, XSS]



from django.forms import Form, ModelForm

from flags import EXTRA_FIELDS, trans
from signals import warning

# Work in progress.
# We aren't trying to be definitive here yet. Just spot stupid things.

class Paranoid(object):

    def __init__(self, data=None, files=None, **kwargs):
        super(Paranoid, self).__init__(data=data, files=files, **kwargs)
        data = data or {}
        extra = [k for k in data if k not in self.fields]
        if extra:
            klass = self.__class__
            msg = (u'%s: %s in %s' %
                   (trans[EXTRA_FIELDS], extra, klass.__name__))
            warning.send(sender=klass, flag=EXTRA_FIELDS,
                         message=msg, values=extra)

    def clean(self):
        super(Paranoid, self).clean()
        for k, v in self.cleaned_data.values():
            # Spot SQL injection attempts.
            # Spot XSS attempt.
            pass


class ParanoidForm(Paranoid, Form):
    pass


class ParanoidModelForm(Paranoid, ModelForm):
    pass



from django.conf import settings
from django.contrib.sessions.middleware import SessionMiddleware
from django.contrib.sessions.backends.cache import SessionStore as Base
from django.core.cache import cache

from signals import warning
from flags import trans, SESSION_CHANGED

KEY_PREFIX = 'django_paranoid.sessions:'
DATA_PREFIX = '%sdata' % KEY_PREFIX
META_KEYS = ['REMOTE_ADDR']


class SessionStore(Base):


    def __init__(self, request=None, session_key=None):
        self._cache = cache
        self.request = request
        super(SessionStore, self).__init__(session_key)

    @property
    def cache_key(self):
        return KEY_PREFIX + self._get_or_create_session_key()

    def load(self):
        return super(SessionStore, self).load()

    def save(self, must_create=False):
        data = self._get_session(no_load=must_create)
        data.setdefault(DATA_PREFIX, {})
        for k in META_KEYS:
            data[DATA_PREFIX]['meta:%s' % k] = self.request.META.get(k, '')
        return super(SessionStore, self).save(must_create=must_create)

    def create(self):
        # Having Django wildly create new sessions is bizarre. Let's
        # instead ensure they are unique. Maybe.
        pass

    def request_data(self):
        return self._get_session(no_load=False)[DATA_PREFIX]

    def check_request_data(self, request):
        data = self._get_session()
        for k in META_KEYS:
            saved = data[DATA_PREFIX]['meta:%s' % k]
            current = request.META.get(k, '')
            if saved and saved != current:
                values = [saved, current]
                msg = msg = (u'%s: %s' % (trans[SESSION_CHANGED], values))
                warning.send(sender=self, flag=SESSION_CHANGED,
                             message=msg, values=values)


class ParanoidSessionMiddleware(SessionMiddleware):

    def process_request(self, request):
        if settings.SESSION_ENGINE != 'django_paranoia.sessions':
            raise ValueError('SESSION_ENGINE must be django_paranoia.sessions')

        session_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, None)
        request.session = SessionStore(request=request,
                                       session_key=session_key)



import logging

from django.utils.importlib import import_module

from signals import warning

log = logging.getLogger('paranoia')


def config(reporters=None, *args, **kw):
    if not reporters:
        try:
            from django.conf import settings
            reporters = getattr(settings, 'DJANGO_PARANOIA_REPORTERS', [])
        except ImportError:
            # This can occur when running the tests, because at this time
            # the settings module isn't created. TODO: fix this.
            return

    for reporter in reporters:
        try:
            to = import_module(reporter).report
        except ImportError:
            log.error('Failed to register the reporter: %s' % reporter)
            continue

        warning.connect(to, dispatch_uid='django-paranoia-%s' % reporter)



from setuptools import setup


setup(
    name='django-paranoia',
    version='0.1',
    description='OWASP detection point reporting for Django',
    long_description=open('README.rst').read(),
    author='Andy McKay',
    author_email='andym@mozilla.com',
    license='BSD',
    install_requires=['Django'],
    packages=['django_paranoia',
              'django_paranoia/reporters'],
    url='https://github.com/andymckay/django-paranoia',
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Framework :: Django'
    ]
)
